@@ -921,6 +921,132 @@ def process_vpr(procstatus, dscfg, radar_list=None):
         return None, None
 
     refl_corr = pyart.correct.correct_vpr(radar, refl_field=refl_field)
+    # User defined variables
+    nvalid_min = dscfg.get('nvalid_min', 20)
+    angle_min = dscfg.get('angle_min', 0.)
+    angle_max = dscfg.get('angle_max', 4.)
+    ml_thickness_min = dscfg.get('ml_thickness_min', 200)
+    ml_thickness_max = dscfg.get('ml_thickness_max', 800)
+    ml_thickness_step = dscfg.get('ml_thickness_step', 200)
+    iso0_max = dscfg.get('iso0_max', 5000.)
+    ml_top_diff_max = dscfg.get('ml_top_diff_max', 200)
+    ml_top_step = dscfg.get('ml_top_step', 200)
+    ml_peak_min = dscfg.get('ml_peak_min', 1.)
+    ml_peak_max = dscfg.get('ml_peak_max', 6.)
+    ml_peak_step = dscfg.get('ml_peak_step', 1.)
+    dr_min = dscfg.get('dr_min', -6.)
+    dr_max = dscfg.get('dr_max', -1.5)
+    dr_step = dscfg.get('dr_step', 1.5)
+    dr_default = dscfg.get('dr_default', -4.5)
+    dr_alt = dscfg.get('dr_alt', 800.)
+    h_max = dscfg.get('h_max', 6000.)
+    h_res = dscfg.get('h_res', 1.)
+    max_weight = dscfg.get('max_weight', 9.)
+    rmin_obs = dscfg.get('rmin_obs', 5000.)
+    rmax_obs = dscfg.get('rmax_obs', 150000.)
+    use_ml = dscfg.get('use_ml', False)
+    ml_datatypedescr = dscfg.get('ml_datatype', None)
+    z_datatypedescr = dscfg.get('z_datatype', None)
+    vpr_theo_datatypedescr = dscfg.get('vpr_theo_datatype', None)
+    vpr_memory_max = dscfg.get('vpr_memory_max', 0.)
+    filter_vpr_memory_max = dscfg.get('filter_vpr_memory_max', 0.)
+    weight_mem = dscfg.get('weight_mem', 0.75)
+
+    iso0 = None
+    if use_ml:
+        if (ml_datatypedescr is None or dscfg['loadbasepath'] is None
+                or dscfg['loadname'] is None):
+            warn('unable to find files containing melting layer information')
+            iso0 = None
+        else:
+            flist = get_file_list(
+                ml_datatypedescr, [dscfg['timeinfo']], [dscfg['timeinfo']],
+                dscfg)
+            if not flist:
+                warn('unable to find files containing'
+                     ' melting layer information')
+                iso0 = None
+            else:
+                radar_ml = pyart.io.read_cfradial(flist[0])
+                if radar_ml is None:
+                    warn('Unable to use retrieved melting layer data')
+                    iso0 = None
+                else:
+                    print('Using file {} with melting layer information'.format(
+                        flist[0]))
+                    iso0 = np.ma.mean(
+                        radar_ml.fields['melting_layer_height']['data'][:, 1])
+                    ml_bottom = np.ma.mean(
+                        radar_ml.fields['melting_layer_height']['data'][:, 0])
+                    ml_thickness = iso0-ml_bottom
+                    ml_thickness_min = ml_thickness-ml_thickness_step
+                    ml_thickness_max = ml_thickness+ml_thickness_step
+
+    radar_mem_list = None
+    if vpr_memory_max > 0:
+        if (z_datatypedescr is None or dscfg['loadbasepath'] is None
+                or dscfg['loadname'] is None):
+            warn('unable to find files with azimuthal reflectivity average')
+        else:
+            # get previous reflectivity files but do not include current one
+            flist = get_file_list(
+                z_datatypedescr,
+                [dscfg['timeinfo']-datetime.timedelta(minutes=vpr_memory_max)],
+                [dscfg['timeinfo']-datetime.timedelta(seconds=1)], dscfg)
+            if not flist:
+                warn('unable to find files containing reflectivity')
+            else:
+                radar_mem_list = []
+                for fname in flist:
+                    radar_vpr = pyart.io.read_cfradial(fname)
+                    if radar_vpr is None:
+                        warn('Unable to use azimuthal reflectivity average')
+                        continue
+                    print('Using file {} with azimuthal averaged refl'.format(
+                          fname))
+                    radar_mem_list.append(radar_vpr)
+                if not radar_mem_list:
+                    radar_mem_list = None
+
+    vpr_theo_dict_mem = None
+    if filter_vpr_memory_max > 0.:
+        if (vpr_theo_datatypedescr is None or dscfg['loadbasepath'] is None
+                or dscfg['loadname'] is None):
+            warn('unable to find files with theoretical VPR')
+        else:
+            # get previous VPR retrieved files but do not include current one
+            flist = get_file_list(
+                vpr_theo_datatypedescr,
+                [dscfg['timeinfo']
+                 - datetime.timedelta(minutes=filter_vpr_memory_max)],
+                [dscfg['timeinfo']-datetime.timedelta(seconds=1)], dscfg)
+            if not flist:
+                warn('unable to find files containing retrieved VPR')
+            else:
+                height, _, vals = read_rhi_profile(flist[-1], labels=['Znorm'])
+                vpr_theo_dict_mem = {
+                    'value': vals[:, 0],
+                    'altitude': height}
+                print('Using file {} with previously retrieved VPR'.format(
+                      flist[-1]))
+
+    corr_refl_field = 'corrected_reflectivity'
+    corr_field = 'vpr_correction'
+    refl_corr, vpr_corr, vpr_theo_dict, radar_rhi = pyart.correct.correct_vpr(
+        radar, nvalid_min=nvalid_min, angle_min=angle_min,
+        angle_max=angle_max, ml_thickness_min=ml_thickness_min,
+        ml_thickness_max=ml_thickness_max,
+        ml_thickness_step=ml_thickness_step, iso0_max=iso0_max,
+        ml_top_diff_max=ml_top_diff_max, ml_top_step=ml_top_step,
+        ml_peak_min=ml_peak_min, ml_peak_max=ml_peak_max,
+        ml_peak_step=ml_peak_step, dr_min=dr_min, dr_max=dr_max,
+        dr_step=dr_step, dr_default=dr_default, dr_alt=dr_alt, h_max=h_max,
+        h_res=h_res, max_weight=max_weight, rmin_obs=rmin_obs,
+        rmax_obs=rmax_obs, iso0=iso0, weight_mem=weight_mem,
+        vpr_theo_dict_mem=vpr_theo_dict_mem, radar_mem_list=radar_mem_list,
+        refl_field=refl_field, corr_refl_field=corr_refl_field,
+        corr_field=corr_field, temp_field=temp_field, iso0_field=iso0_field,
+        temp_ref=temp_ref)
 
     # prepare for exit
     new_dataset = {'radar_out': deepcopy(radar)}
